﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace projectDoubleDeckerBus
{
    public class DrawningDoubleDeckerBus
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityDoubleDeckerBus? EntityDoubleDeckerBus { get; private set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// 
        /// Левая координата прорисовки автобуса
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автобуса
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки автобуса
        /// </summary>
        private readonly int _busWidth = 172;
        /// <summary>
        /// Высота прорисовки автобуса
        /// </summary>
        private readonly int _busHeight = 113;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="additionalColor">Дополнительный цвет</param>     
        /// <param name="headlight">Признак наличия фары</param>
        /// <param name="floor">Признак наличия этажа</param>
        /// <param name="tailpipe">Признак наличия выхлопа</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена,

        public bool Init(int speed, double weight, Color bodyColor, Color
        additionalColor, bool headlight, bool floor, bool tailpipe, int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            EntityDoubleDeckerBus = new EntityDoubleDeckerBus();
            EntityDoubleDeckerBus.Init(speed, weight, bodyColor, additionalColor, headlight, floor, tailpipe);
            /// Проверка на вместимость обьекта в рамки сцены
            /// </summary>
            if ((_busWidth >= _pictureWidth) || (_busHeight >= _pictureHeight))
            {
                Console.WriteLine("проверка не пройдена, нельзя создать объект в этих размерах");
                return false;
            }
            else
                Console.WriteLine("объект создан");
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (((x >= 0) && (x + _busWidth > _pictureWidth)) || ((y >= 0) && (y + _busHeight > _pictureHeight)))
            {
                _startPosX = _pictureWidth - _busWidth;
                _startPosY = _pictureHeight - _busHeight;
            }

            else
            {
                _startPosX = x;
                _startPosY = y;
            }
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (EntityDoubleDeckerBus == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case Direction.Left:
                    if (_startPosX - EntityDoubleDeckerBus.Step > 0)
                    {
                        _startPosX -= (int)EntityDoubleDeckerBus.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - EntityDoubleDeckerBus.Step > 0)
                    {
                        _startPosY -= (int)EntityDoubleDeckerBus.Step;
                    }
                    break;
                // вправо
                case Direction.Right:
                    if (_startPosX + _busWidth + EntityDoubleDeckerBus.Step < _pictureWidth)
                    {
                        _startPosX += (int)EntityDoubleDeckerBus.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _busHeight + EntityDoubleDeckerBus.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityDoubleDeckerBus.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityDoubleDeckerBus == null)
            {
                return;
            }

            int windowSize = 15;
            Brush brBlack = new SolidBrush(Color.Black);
            Brush brRed = new SolidBrush(Color.Red);

            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityDoubleDeckerBus.AdditionalColor);

            Brush br = new SolidBrush(EntityDoubleDeckerBus?.BodyColor ?? Color.Black);
            g.FillRectangle(br, _startPosX + 40, _startPosY + 34, 130, 52);

            g.DrawEllipse(pen, _startPosX + 46, _startPosY + 50, windowSize, windowSize); 
            g.DrawEllipse(pen, _startPosX + 62, _startPosY + 50, windowSize, windowSize); 
            g.DrawEllipse(pen, _startPosX + 100, _startPosY + 50, windowSize, windowSize); 

            g.DrawEllipse(pen, _startPosX + 115, _startPosY + 50, windowSize, windowSize); 
            g.DrawEllipse(pen, _startPosX + 130, _startPosY + 50, windowSize, windowSize); 
            g.DrawEllipse(pen, _startPosX + 145, _startPosY + 50, windowSize, windowSize);           

            //колеса
            g.FillRectangle(brBlack, _startPosX + 80, _startPosY + 46, 15, 40);
            g.FillEllipse(brBlack, _startPosX + 50, _startPosY + 75, 20, 20);
            g.FillEllipse(brBlack, _startPosX + 140, _startPosY + 75, 20, 20);

            Brush brBlue = new SolidBrush(Color.LightBlue);
            g.FillEllipse(brBlue, _startPosX + 46, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 62, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 100, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 115, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 130, _startPosY + 50, windowSize, windowSize);
            g.FillEllipse(brBlue, _startPosX + 145, _startPosY + 50, windowSize, windowSize);

            if (EntityDoubleDeckerBus.Floor)
            {
                g.DrawEllipse(pen, _startPosX + 46, _startPosY + 10, windowSize, windowSize); 
                g.DrawEllipse(pen, _startPosX + 62, _startPosY + 10, windowSize, windowSize); 
                g.DrawEllipse(pen, _startPosX + 100, _startPosY + 10, windowSize, windowSize); 

                g.DrawEllipse(pen, _startPosX + 115, _startPosY + 10, windowSize, windowSize); 
                g.DrawEllipse(pen, _startPosX + 130, _startPosY + 10, windowSize, windowSize); 
                g.DrawEllipse(pen, _startPosX + 145, _startPosY + 10, windowSize, windowSize); 

                g.FillRectangle(additionalBrush, _startPosX + 40,
                _startPosY + 1, 130, 32);

                Brush brBlue1 = new SolidBrush(Color.LightBlue);
                g.FillEllipse(brBlue, _startPosX + 46, _startPosY + 10, windowSize, windowSize);
                g.FillEllipse(brBlue, _startPosX + 62, _startPosY + 10, windowSize, windowSize);
                g.FillEllipse(brBlue, _startPosX + 100, _startPosY + 10, windowSize, windowSize);
                g.FillEllipse(brBlue, _startPosX + 115, _startPosY + 10, windowSize, windowSize);
                g.FillEllipse(brBlue, _startPosX + 130, _startPosY + 10, windowSize, windowSize);
                g.FillEllipse(brBlue, _startPosX + 145, _startPosY + 10, windowSize, windowSize);
            }

            if (EntityDoubleDeckerBus.Headlight)
            {
                g.FillEllipse(additionalBrush, _startPosX + 162, _startPosY + 78, 10, 10);
            }

            if (EntityDoubleDeckerBus.Tailpipe)
            {
                g.FillRectangle(additionalBrush, _startPosX, _startPosY + 76, 40, 10);

            }
        }   
    }
}
